@@ -12,14 +12,17 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import logging
+import sys
 from argparse import ArgumentParser, FileType
 from contextlib import contextmanager
 
+from ufo2ft import CFFOptimization
+from ufo2ft.featureWriters import loadFeatureWriterFromString
+
 from fontmake import __version__
 from fontmake.errors import FontmakeError
 from fontmake.font_project import INTERPOLATABLE_OUTPUTS, FontProject
-from ufo2ft import CFFOptimization
-from ufo2ft.featureWriters import loadFeatureWriterFromString
 
 
 def _loadFeatureWriters(parser, specs):
@@ -122,6 +125,7 @@ def main(args=None):
         choices=(
             "ufo",
             "otf",
+            "otf-cff2",
             "ttf",
             "ttf-interpolatable",
             "otf-interpolatable",
@@ -254,6 +258,13 @@ def main(args=None):
         help="Maximum approximation error for cubic to quadratic conversion "
         "measured in EM",
     )
+    contourGroup.add_argument(
+        "-f",
+        "--flatten-components",
+        dest="flatten_components",
+        action="store_true",
+        help="Flatten nested components to single level.",
+    )
     contourGroup.add_argument(
         "-a",
         "--autohint",
@@ -278,6 +289,14 @@ def main(args=None):
         help="0 disables all optimizations; 1 specializes the CFF charstring "
         "operators; 2 (default) also enables subroutinization",
     )
+    contourGroup.add_argument(
+        "--subroutinizer",
+        default=None,
+        choices=["compreffor", "cffsubr"],
+        help="name of the library to use for compressing CFF charstrings. "
+        "Choose between: %(choices)s. By default compreffor is used for CFF 1, "
+        "and cffsubr for CFF2. NOTE: compreffor doesn't support CFF2.",
+    )
     contourGroup.add_argument(
         "--no-optimize-gvar",
         dest="optimize_gvar",
@@ -323,7 +342,7 @@ def main(args=None):
     feaCompilerGroup = layoutGroup.add_mutually_exclusive_group(required=False)
     feaCompilerGroup.add_argument(
         "--mti-source",
-        help="Path to mtiLib .txt feature definitions (use instead of FEA)",
+        help="mtiLib feature definition .plist file path (use instead of FEA)",
     )
 
     glyphnamesGroup = parser.add_mutually_exclusive_group(required=False)
@@ -413,6 +432,7 @@ def main(args=None):
                     "[mutatormath] extra"
                 )
 
+    PRINT_TRACEBACK = args.get("verbose", "INFO") == "DEBUG"
     try:
         project = FontProject(
             timing=args.pop("timing"),
@@ -460,12 +480,11 @@ def main(args=None):
             ufo_paths, is_instance=args.pop("masters_as_instances"), **args
         )
     except FontmakeError as e:
-        import sys
-
-        sys.exit("fontmake: error: %s" % e)
+        if PRINT_TRACEBACK:
+            logging.exception(e)
+            sys.exit(1)
+        sys.exit(f"fontmake: Error: {str(e)}")
 
 
 if __name__ == "__main__":
-    import sys
-
     sys.exit(main())
